from __future__ import print_function, division, absolute_import

from . import helm, misc, culgoora, gbsix, mrt, nvss, parkes, txs, wenss
from . import three_c, three_cr, four_c, six_c, seven_c,paper,vlss



from __future__ import print_function, division, absolute_import

import unittest, aipy._src.culgoora as h, aipy as a, numpy as n

class TestCulgooraCatalog(unittest.TestCase):
    def setUp(self):
        self.cat = h.CulgooraCatalog()
        self.cat.fromfile(h.CULGOORAFILE)
    def test_spotcheck(self):
        for srcname in self.cat:
            src = self.cat[srcname]
            self.assertEqual(len(src.srcshape), 3)
            self.assertEqual(src.srcshape[0], 0)
            self.assertEqual(src.srcshape[1], 0)
            self.assertEqual(src.srcshape[2], 0)
            self.assertTrue(src.mfreq in [.080, .160])
    def test_metadata(self):
        md = self.cat.get_metadata()
        for src in md:
            self.assertEqual(len(md[src]), 3)
            jy080, jy160, index = md[src]
            if index is None:
                self.assertTrue(jy080 is None or jy160 is None)
            else:
                self.assertNotEqual(jy080, None)
                self.assertNotEqual(jy160, None)
                #self.assertAlmostEqual(index, n.log2(jy160/jy080), 1)


if __name__ == '__main__':
    unittest.main()



"""
Provides interfaces to Healpix_cxx, which was developed at the
Max-Planck-Institut fuer Astrophysik and financially supported by the
Deutsches Zentrum fuer Luft- und Raumfahrt (DLR).
Adds data to the HealpixBase class using numpy arrays, and interfaces to
FITS files using astropy/pyfits.
"""

from __future__ import print_function, division, absolute_import

import numpy as np
from . import utils
try:
    from astropy.io import fits as pyfits
except ImportError:
    import pyfits

import healpy

default_fits_format_codes = {
    np.bool_:'L', np.uint8:'B', np.int16:'I', np.int32:'J', np.int64:'K',
    np.float32:'E', np.float64:'D', np.complex64:'C', np.complex128:'M'
}

def mk_arr(val, dtype=np.double):
    if type(val) is np.ndarray: return val.astype(dtype)
    return np.array(val, dtype=dtype).flatten()

HEALPIX_MODES = ('RING','NEST')

class HealpixBase(object):
    """Functionality related to the HEALPix pixelisation."""
    def __init__(self, nside=1, scheme='RING'):
        self._nside = nside
        self._scheme = scheme
    def npix2nside(self, npix):
        """Convert number of pixels to number of sides."""
        return healpy.npix2nside(npix)
    def nest_ring_conv(self, px, scheme):
        """Translate an array of pixel numbers to index data in the scheme specified in 'scheme' ('NEST' or 'RING').  Returns px."""
        mode = {'RING':healpy.nest2ring, 'NEST':healpy.ring2nest}
        if scheme != self._scheme: px = mode[scheme](self._nside, px)
        self._scheme = scheme
        return px
    def set_nside_scheme(self, nside=None, scheme=None):
        """Adjust Nside and Scheme ('RING' or 'NEST')."""
        if nside is not None:
            pow2 = np.log2(nside)
            assert(pow2 == np.around(pow2))
            self._nside = nside
        if scheme is not None:
            assert(scheme in HEALPIX_MODES)
            self._scheme = scheme
    def crd2px(self, c1, c2, c3=None, interpolate=False):
        """Convert 1 dimensional arrays of input coordinates to pixel indices. If only c1,c2 provided, then read them as th,phi.  If c1,c2,c3 provided, read them as x,y,z. If interpolate is False, return a single pixel coordinate.  If interpolate is True, return px,wgts where each entry in px contains the 4 pixels adjacent to the specified location, and wgt contains the 4 corresponding weights of those pixels."""
        is_nest = (self._scheme == 'NEST')
        if not interpolate:
            if c3 is None: # th/phi angle mode
                px = healpy.ang2pix(self._nside, c1, c2, nest=is_nest)
            else: # x,y,z mode
                px = healpy.vec2pix(self._nside, c1, c2, c3, nest=is_nest)
            return px
        else:
            if c3 is not None: # need to translate xyz to th/phi
                c1,c2 = healpy.vec2ang(np.array([c1,c2,c3]).T)
            px,wgts = healpy.get_interp_weights(self._nside, c1, c2, nest=is_nest)
            return px.T, wgts.T
    def px2crd(self, px, ncrd=3):
        """Convert a 1 dimensional input array of pixel numbers to the type of coordinates specified by ncrd.  If ncrd=3 (default), the returned array will have (x,y,z) for each pixel.  Otherwise if ncrd=2, the returned array will have (theta,phi) for each pixel."""
        is_nest = (self._scheme == 'NEST')
        assert(ncrd in (2,3))
        if ncrd == 2: # th/phi mode
            th,phi = healpy.pix2ang(self._nside, px, nest=is_nest)
            return th, phi
        else: # ncrd == 3 -> vec mode
            x,y,z = healpy.pix2vec(self._nside, px, nest=is_nest)
            return x,y,z
    def order(self):
        """Return the order parameter."""
        return healpy.nside2order(self._nside)
    def nside(self):
        """Return the Nside parameter."""
        return self._nside
    def npix(self):
        """Return the number of pixels in the map."""
        return healpy.nside2npix(self._nside)
    def scheme(self):
        """Return the scheme of the map ('NEST' or 'RING')."""
        return self._scheme

class Alm(object):
    '''Holds coefficients for spherical harmonics up to the specified order, and generates a real-valued HealpixMap from them.  Individual (l,m) coefficients can be accessed by alm[l,m].'''
    def __init__(self, lmax, mmax, dtype=np.complex128):
        assert(lmax >= mmax)
        self._alm = healpy.Alm()
        self._lmax = lmax
        self._mmax = mmax
        self.dtype = dtype
        self.set_to_zero()
    def size(self):
        '''Return the number of Alm coefficients.'''
        return self._alm.getsize(self._lmax,self._mmax)
    def set_to_zero(self):
        """Clear all coefficients."""
        data = np.zeros(self.size(), dtype=self.dtype)
        self.set_data(data)
    def lmax(self):
        """Return the maximum L."""
        return self._lmax
    def mmax(self):
        """Return the maximum M."""
        return self._mmax
    def __getitem__(self, lm):
        l,m = lm
        i = self._alm.getidx(self._lmax, l, m)
        return self.data[i]
    def __setitem__(self, lm, val):
        l,m = lm
        i = self._alm.getidx(self._lmax, l, m)
        self.data[i] = val
    def to_map(self, nside, pixwin=False, fwhm=0.0, sigma=None, 
            pol=True, verbose=False):
        '''Return data for the Healpix map in 'RING' mode with the specified 
        nside (power of 2) generated by these Alm coefficients.

        Parameters
        ----------
        nside : int, scalar
          The nside of the output map.
        pixwin : bool, optional
          Smooth the alm using the pixel window functions. Default: False.
        fwhm : float, scalar, optional
          The fwhm of the Gaussian used to smooth the map (applied on alm)
          [in radians]
        sigma : float, scalar, optional
          The sigma of the Gaussian used to smooth the map (applied on alm)
          [in radians]
        pol : bool, optional
          If True, assumes input alms are TEB. Output will be TQU maps.
          (input must be 1 or 3 alms)
          If False, apply spin 0 harmonic transform to each alm.
          (input can be any number of alms)
          If there is only one input alm, it has no effect. Default: True.

        Returns
        -------
        maps : array or list of arrays
          A Healpix map in RING scheme at nside or a list of T,Q,U maps (if
          polarized input)'''
        return healpy.alm2map(self.get_data(), nside, 
            lmax=self._lmax, mmax=self._mmax,
            pixwin=pixwin, fwhm=fwhm, sigma=sigma, pol=pol, verbose=verbose)
    def from_map(self, data, iter=3, pol=True, use_weights=False, gal_cut=0):
        '''Set the coefficients of this Alm object (with its specified 
        lmax and mmax) from the data of a HealpixMap in 'RING' mode.

        Parameters
        ----------
        data : array-like, shape (Npix,) 
          The input map.
        iter : int, scalar, optional
          Number of iteration (default: 3)
        pol : bool, optional
          If True, assumes input maps are TQU. Output will be TEB alm's.
          (input must be 1 or 3 maps)
          If False, apply spin 0 harmonic transform to each map.
          (input can be any number of maps)
          If there is only one input map, it has no effect. Default: True.
        use_weights: bool, scalar, optional
          If True, use the ring weighting. Default: False.
        gal_cut : float [degrees]
          pixels at latitude in [-gal_cut;+gal_cut] are not taken into account'''
        data = healpy.map2alm(data, lmax=self._lmax, mmax=self._mmax, iter=iter,
            pol=pol, use_weights=use_weights, gal_cut=gal_cut)
        self.set_data(data)
    def lm_indices(self):
        """Return the L and M indices of the coefficients contained in Alm, in the order that they are returned by get_data()."""
        return np.array([self._alm.getlm(self._lmax, i) for i in range(self.size)])
    def get_data(self):
        """Return all of the coefficients contained in Alm, in the order indexed by lm_indices()."""
        return self.data
    def set_data(self, data):
        """Set all of coefficients contained in Alm, in the order indexed by lm_indices()."""
        assert(data.size == self.size())
        self.data = data.astype(self.dtype)

class HealpixMap(HealpixBase):
    """Collection of utilities for mapping data on a sphere.  Adds a data map
    to the infrastructure in _healpix.HealpixBase."""
    def __init__(self, *args, **kwargs):
        dtype = kwargs.pop('dtype', np.double)
        interp = kwargs.pop('interp', False)
        fromfits = kwargs.pop('fromfits', None)
        HealpixBase.__init__(self, *args, **kwargs)
        self._use_interpol = interp
        if fromfits is None:
            m = np.zeros((self.npix(),), dtype=dtype)
            self.set_map(m, scheme=self.scheme())
        else: self.from_fits(fromfits)
    def set_interpol(self, onoff):
        """Choose whether __getitem___ (i.e. HealpixMap[crd]) returns an
        interpolated value or just nearest pixel.  Default upon creating a
        HealpixMap is to not interpolate."""
        self._use_interpol = onoff
    def set_map(self, data, scheme="RING"):
        """Assign data to HealpixMap.map.  Infers Nside from # of pixels via
        Npix = 12 * Nside**2."""
        try:
            assert(data.ndim == 1)
            nside = self.npix2nside(data.shape[0])
        except(AssertionError,ValueError):
            raise ValueError("Data must be a 1 dim array with 12*N**2 entries.")
        self.set_nside_scheme(nside, scheme)
        self.map = data
    def get_map(self):
        """Return Healpix data as a 1 dimensional numpy array."""
        return self.map
    def change_scheme(self, scheme):
        """Reorder the pixels in map to be "RING" or "NEST" ordering."""
        assert(scheme in ["RING", "NEST"])
        if scheme == self.scheme(): return
        i = self.nest_ring_conv(np.arange(self.npix()), scheme)
        self[i] = self.map
        self.set_nside_scheme(self.nside(), scheme)
    def __getitem__(self, crd):
        """Access data on a sphere via hpm[crd].
        crd = either 1d array of pixel indices, (th,phi), or (x,y,z), where
        th,phi,x,y,z are numpy arrays of coordinates."""
        if type(crd) is tuple:
            crd = [mk_arr(c, dtype=np.double) for c in crd]
            if self._use_interpol:
                px,wgts = self.crd2px(*crd, **{'interpolate':1})
                return np.sum(self.map[px] * wgts, axis=-1)
            else: px = self.crd2px(*crd)
        else: px = mk_arr(crd, dtype=np.int64)
        return self.map[px]
    def __setitem__(self, crd, val):
        """Assign data to a sphere via hpm[crd] = val.  Functionality slightly
        complicated to make repeat coordinates assign sum of values (i.e.
        crd = ([1,1], [2,2]), val = [3,3] will assign 6 to location (1,2).
        crd = either 1d array of pixel indices, (th,phi), or (x,y,z), where
        th,phi,x,y,z are numpy arrays of coordinates."""
        if type(crd) is tuple:
            crd = [mk_arr(c, dtype=np.double) for c in crd]
            px = self.crd2px(*crd)
        else:
            if type(crd) is np.ndarray: assert(len(crd.shape) == 1)
            px = mk_arr(crd, dtype=int)
        if px.size == 1:
            if type(val) is np.ndarray: val = mk_arr(val, dtype=self.map.dtype)
            self.map[px] = val
        else:
            m = np.zeros_like(self.map)
            px = px.reshape(px.size,1)
            cnt = np.zeros(self.map.shape, dtype=np.bool_)
            val = mk_arr(val, dtype=m.dtype)
            utils.add2array(m, px, val)
            utils.add2array(cnt, px, np.ones(val.shape, dtype=np.bool_))
            self.map = np.where(cnt, m, self.map)
    def from_hpm(self, hpm):
        """Initialize this HealpixMap with data from another.  Takes care
        of upgrading or downgrading the resolution, and swaps ordering
        scheme if necessary."""
        if hpm.nside() < self.nside():
            interpol = hpm._use_interpol
            hpm.set_interpol(True)
            px = np.arange(self.npix())
            th,phi = self.px2crd(px, ncrd=2)
            self[px] = hpm[th,phi].astype(self.get_dtype())
        elif hpm.nside() > self.nside():
            px = np.arange(hpm.npix())
            th,phi = hpm.px2crd(px, ncrd=2)
            self[th,phi] = hpm[px].astype(self.get_dtype())
        else:
            if hpm.scheme() == self.scheme():
                self.map = hpm.map.astype(self.get_dtype())
            else:
                i = self.nest_ring_conv(np.arange(self.npix()), hpm.scheme())
                self.map = hpm.map[i].astype(self.get_dtype())
    def from_alm(self, alm):
        """Set data to the map generated by the spherical harmonic
        coefficients contained in alm."""
        self.set_map(alm.to_map(self.nside(), self.scheme()))
    def to_alm(self, lmax, mmax, iter=1):
        """Return an Alm object containing the spherical harmonic components
        of a map (in RING mode) up to the specified lmax,mmax.  Greater
        accuracy can be achieved by increasing iter."""
        assert(self.scheme() == 'RING')
        alm = Alm(lmax,mmax)
        alm.from_map(self.map, iter)
        return alm
    def from_fits(self, filename, hdunum=1, colnum=0):
        """Read a HealpixMap from the specified location in a fits file."""
        hdu = pyfits.open(filename)[hdunum]
        data = hdu.data.field(colnum)
        if not data.dtype.isnative:
            data.dtype = data.dtype.newbyteorder()
            data.byteswap(True)
        scheme= hdu.header['ORDERING'][:4]
        self.set_map(data, scheme=scheme)
    def _set_fits_header(self, hdr):
        hdr.update('PIXTYPE', 'HEALPIX', 'HEALPIX pixelisation')
        scheme = self.scheme()
        if scheme == 'NEST': scheme == 'NESTED'
        hdr.update('ORDERING', scheme,
            'Pixel ordering scheme, either RING or NESTED')
        hdr.update('NSIDE', self.nside(), 'Resolution parameter for HEALPIX')
        hdr.update('FIRSTPIX', 0, "First pixel # (0 based)")
        hdr.update('LASTPIX', self.npix()-1, "Last pixel # (0 based)")
        hdr.update('INDXSCHM', 'IMPLICIT', "Indexing: IMPLICIT or EXPLICIT")
    def get_dtype(self):
        return self.map.dtype
    def to_fits(self, filename, format=None, clobber=True):
        """Write a HealpixMap to a fits file in the fits format specified by
        'format'.  Default uses mapping of numpy types to fits format types
        stored in default_fits_format_codes."""
        if format is None:
            format = default_fits_format_codes[self.get_dtype().type]
        hdu0 = pyfits.PrimaryHDU()
        col0 = pyfits.Column(name='signal', format=format, array=self.map)
        cols = pyfits.ColDefs([col0])
        tbhdu = pyfits.new_table(cols)
        self._set_fits_header(tbhdu.header)
        hdulist = pyfits.HDUList([hdu0, tbhdu])
        hdulist.writeto(filename,clobber=clobber)



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Aaron Parsons
# Licensed under the GPLv3

import numpy as np
import pytest
import aipy._dsp as _dsp


def test_testgrid1D_c():
    buf = np.zeros(32, dtype=np.complex64)
    ind = np.array([5, 10.1, 14.9], dtype=np.float32)
    dat = np.array([1, 1, 1], dtype=np.complex64)
    _dsp.grid1D_c(buf, ind, dat)
    x = np.arange(32)
    ans = (
        np.exp(-((x - 5) ** 2) / (2 * 0.5 ** 2)) / np.sqrt(2 * np.pi * 0.5 ** 2)
        + np.exp(-((x - 10.1) ** 2) / (2 * 0.5 ** 2)) / np.sqrt(2 * np.pi * 0.5 ** 2)
        + np.exp(-((x - 14.9) ** 2) / (2 * 0.5 ** 2)) / np.sqrt(2 * np.pi * 0.5 ** 2)
    )
    assert np.allclose(np.max(np.abs(buf - ans)), 0, atol=1e-6)


def test_testgrid2D_c():
    buf = np.zeros((32, 32), dtype=np.complex64)
    ind = np.array([[5, 5], [10.1, 10.1], [14.5, 15.5]], dtype=np.float32)
    ind1 = ind[:, 0].copy()  # copy necessary b/c not contigous memory otherwise
    ind2 = ind[:, 1].copy()
    dat = np.array([1, 1, 1], dtype=np.complex64)
    _dsp.grid2D_c(buf, ind1, ind2, dat)
    assert np.allclose(buf[5, 5], 0.63661977236758149, atol=1e-6)
    assert np.allclose(
        buf[10, 10], 0.63661977236758149 * np.exp(-2 * (0.1 ** 2 + 0.1 ** 2)), atol=1e-6
    )
    assert np.allclose(
        buf[14, 15], 0.63661977236758149 * np.exp(-2 * (0.5 ** 2 + 0.5 ** 2)), atol=1e-6
    )


def test_testdegrid2D_c():
    buf = np.ones((32, 32), dtype=np.complex64)
    ind = np.array([[5, 5], [10.1, 10.1], [14.5, 15.5]], dtype=np.float32)
    ind1 = ind[:, 0].copy()  # copy necessary b/c not contigous memory otherwise
    ind2 = ind[:, 1].copy()
    dat = np.zeros(ind1.shape, dtype=np.complex64)
    _dsp.degrid2D_c(buf, ind1, ind2, dat)
    assert np.allclose(dat, 1.0)



# -*- coding: utf-8 -*-
# Copyright (c) 2018 Aaron Parsons
# Licensed under the GPLv3

import aipy.dsp as dsp
import numpy as np


def test_kaiser2():
    win = dsp.gen_window(1024, "kaiser2")
    assert np.allclose(win[0], 0.01147993)
    assert np.allclose(win[1], 0.01192681)
    assert np.allclose(win[2], 0.01238142)



from __future__ import print_function, division, absolute_import

import numpy as np
from scipy.special import i0
from ._dsp import *


def tukey(x, L, alpha=0.5):
    """A Tukey, or tapered-cosine, window. See scipy.signal.windows for details. 
    alpha=0 is rectangle, alpha=1 is hann window."""
    if alpha <= 0:
        return np.ones(L, 'd')
    elif alpha >= 1.0:
        return WINDOW_FUNC['hanning'](x, L)

    width = int(np.floor(alpha*(L-1)/2.0))
    x1 = x[0:width+1]
    x2 = x[width+1:L-width-1]
    x3 = x[L-width-1:]

    w1 = 0.5 * (1 + np.cos(np.pi * (-1 + 2.0*x1/alpha/(L-1))))
    w2 = np.ones(x2.shape)
    w3 = 0.5 * (1 + np.cos(np.pi * (-2.0/alpha + 1 + 2.0*x3/alpha/(L-1))))

    w = np.concatenate((w1, w2, w3))

    return w

def barthann(x, L):
    """A Bartlett-Hann window. See scipy.signal.windows for details """
    x = np.arange(0, L)
    fac = np.abs(x / (L - 1.0) - 0.5)
    w = 0.62 - 0.48 * fac + 0.38 * np.cos(2 * np.pi * fac)

    return w

WINDOW_FUNC = {
    'blackman': lambda x,L: .42-.5*np.cos(2*np.pi*x/(L-1))+.08*np.cos(4*np.pi*x/(L-1)),
    'blackman-harris': lambda x,L: .35875 - .48829*np.cos(2*np.pi*x/(L-1)) + .14128*np.cos(4*np.pi*x/(L-1)) - .01168*np.cos(6*np.pi*x/(L-1)),
    'gaussian0.4': lambda x,L: np.exp(-0.5 * ((x - (L-1)/2)/(0.4 * (L-1)/2))**2),
    'kaiser2': lambda x,L: i0(np.pi * 2 * np.sqrt(1-(2*x/(L-1) - 1)**2)) / i0(np.pi * 2),
    'kaiser3': lambda x,L: i0(np.pi * 3 * np.sqrt(1-(2*x/(L-1) - 1)**2)) / i0(np.pi * 3),
    'hamming': lambda x,L: .54 - .46 * np.cos(2*np.pi*x/(L-1)),
    'hanning': lambda x,L: .5 - .5 * np.cos(2*np.pi*x/(L-1)),
    'parzen': lambda x,L: 1 - np.abs(L/2. - x) / (L/2.),
    'none': lambda x,L: 1,
    'tukey': tukey,
    'barthann': barthann
    }

def gen_window(L, window='hamming', **kwargs):
    '''Return the specified window (see WINDOW_FUNC) for a length L.'''
    return np.fromfunction(lambda x: WINDOW_FUNC[window](x,L,**kwargs), (L,))



# -*- coding: utf-8 -*-
# Copyright (c) 2009 Aaron Parsons
# Licensed under the GPLv3

import re
import pytest
import numpy as np

import aipy
from aipy.miriad import ij2bl


@pytest.fixture(scope="function")
def chan_cases():
    nchan = 256
    cases = {
        "all": [np.arange(nchan)],
        "0,1,2,3": [np.array([0]), np.array([1]), np.array([2]), np.array([3])],
        "0_20": [np.arange(21)],
        "0_20_4": [np.arange(0, 21, 4)],
        "5,7,40_50": [np.array([5]), np.array([7]), np.arange(40, 51)],
    }
    yield nchan, cases
    return


@pytest.fixture(scope="function")
def test_strs():
    yield [
        "src1=jys",
        "src2=jys/100",
        "src3=jys/100/1",
        "src4=jys//1",
        "(src5/src6)=jys",
        "(src7/src8)=jys/(100/200)",
        "(src9/src10)=jys/(100/200)/(1/2)",
        "(src11/src12)=jys/(100/200)/1",
        "(src13/src14/src15)=(jys/index)//1",
    ]
    return


def test_parse_ants():
    """Test aipy.scripting.parse_ants()"""
    nants = 4
    cases = {
        "all": [],
        "auto": [("auto", 1)],
        "cross": [("auto", 0)],
        "0_1": [(ij2bl(0, 1), 1)],
        "0_1,1_2": [(ij2bl(0, 1), 1), (ij2bl(1, 2), 1)],
        "0x_1x": [(ij2bl(0, 1), 1, "xx")],
        "(0x,0y)_1x": [(ij2bl(0, 1), 1, "xx"), (ij2bl(0, 1), 1, "yx")],
        "(0,1)_2": [(ij2bl(0, 2), 1), (ij2bl(1, 2), 1)],
        "0_(1,2)": [(ij2bl(0, 1), 1), (ij2bl(0, 2), 1)],
        "(0,1)_(2,3)": [
            (ij2bl(0, 2), 1),
            (ij2bl(0, 3), 1),
            (ij2bl(1, 2), 1),
            (ij2bl(1, 3), 1),
        ],
        "0_(1,-2)": [(ij2bl(0, 1), 1), (ij2bl(0, 2), 0)],
        "(-0,1)_(2,-3)": [
            (ij2bl(0, 2), 0),
            (ij2bl(0, 3), 0),
            (ij2bl(1, 2), 1),
            (ij2bl(1, 3), 0),
        ],
        "0,1,all": [],
    }
    for i in range(nants):
        cases[str(i)] = list(map(lambda x: (ij2bl(x, i), 1), range(nants)))
        cases["-" + str(i)] = list(map(lambda x: (ij2bl(x, i), 0), range(nants)))
    # inelegantly paste on the new pol parsing flag on the above tests
    # XXX really should add some new tests for the new pol parsing
    for k in cases:
        cases[k] = [(v + (-1,))[:3] for v in cases[k]]
    for ant_str in cases:
        assert aipy.scripting.parse_ants(ant_str, nants) == cases[ant_str]

    with pytest.raises(ValueError):
        aipy.scripting.parse_ants("(0_1)_2", nants)

    return


def test_parse_srcs():
    """Test aipy.scripting.parse_srcs()"""
    cat = "misc,helm"
    src_opt = "cyg,cas,12_40,12:00_40:00,12:00:00.0_40:00:00.0,bug_man"
    srclist, cutoff, catalogs = aipy.scripting.parse_srcs(src_opt, cat)
    assert srclist[0] == "cyg"
    assert srclist[1] == "cas"
    assert srclist[2].src_name == "12_40"
    assert srclist[3].src_name == "12:00_40:00"
    assert srclist[4].src_name == "12:00:00.0_40:00:00.0"
    assert srclist[5] == "bug_man"
    return


def test_parse_chans_concat(chan_cases):
    """Test aipy.scripting.parse_chans() - concatenate"""
    nchan, cases = chan_cases
    for case in cases:
        chans = aipy.scripting.parse_chans(case, nchan, concat=True)
        assert np.allclose(chans, np.concatenate(cases[case]))

    with pytest.raises(AssertionError):
        aipy.scripting.parse_chans("0_1_2_3", nchan)

    return


def test_parse_chans_noconcat(chan_cases):
    """Test aipy.scripting.parse_chans() - without concatenate"""
    nchan, cases = chan_cases
    for case in cases:
        chans = aipy.scripting.parse_chans(case, nchan, concat=False)
        for ch1, ch2 in zip(chans, cases[case]):
            assert np.allclose(ch1, ch2)
    return


def test_name():
    """Test aipy.scripting.name matching"""
    regex = re.compile(aipy.scripting.name)
    assert regex.match("src1").groups() == ("src1",)
    assert regex.match("1").groups() == ("1",)
    assert regex.match("1,1").groups() == ("1",)
    assert regex.match("1=1").groups() == ("1",)
    assert regex.match("1/1").groups() == ("1",)
    assert regex.match("1(1").groups() == ("1",)
    assert regex.match("1)1").groups() == ("1",)
    assert regex.match(")1") is None
    return


def test_grp():
    """Test aipy.scripting.grp matching"""
    regex = re.compile(aipy.scripting.grp)
    assert regex.match("src1").groups()[0] == "src1"
    assert regex.match("src1/src2").groups()[0] == "src1"
    assert regex.match("(src1/src2)").groups()[0] == "(src1/src2)"
    assert regex.match("(1/2/3)").groups()[0] == "(1/2/3)"
    assert regex.match("(src1/)") is None
    assert regex.match("(src1/src2") is None
    return


def test_prm_rgx(test_strs):
    """Test aipy.scripting.prm_rgx matching"""
    regex = aipy.scripting.prm_rgx
    for t in test_strs:
        assert regex.match(t).groups()[0] == t
    return


def test_parse_prms(test_strs):
    """Test aipy.scripting.parse_prms()"""
    t = ",".join(test_strs)
    prms = aipy.scripting.parse_prms(t)
    assert prms["src1"]["jys"] == (None, None)
    assert prms["src2"]["jys"] == (100.0, None)
    assert prms["src3"]["jys"] == (100.0, 1.0)
    assert prms["src4"]["jys"] == (None, 1.0)
    assert prms["src5"]["jys"] == (None, None)
    assert prms["src6"]["jys"] == (None, None)
    assert prms["src7"]["jys"] == (100.0, None)
    assert prms["src8"]["jys"] == (200.0, None)
    assert prms["src9"]["jys"] == (100.0, 1.0)
    assert prms["src10"]["jys"] == (200.0, 2.0)
    assert prms["src11"]["jys"] == (100.0, 1.0)
    assert prms["src12"]["jys"] == (200.0, 1.0)
    assert prms["src13"]["jys"] == (None, 1.0)
    assert prms["src13"]["index"] == (None, 1.0)
    assert prms["src14"]["jys"] == (None, 1.0)
    assert prms["src14"]["index"] == (None, 1.0)
    assert prms["src15"]["jys"] == (None, 1.0)
    assert prms["src15"]["index"] == (None, 1.0)

    with pytest.raises(AssertionError):
        aipy.scripting.parse_prms("(a/b)=(c/d)/(1/2)/(3/4)")

    t = "(1/2/3)=jys,(2/3)=index"
    prms = aipy.scripting.parse_prms(t)
    assert len(prms["1"]) == 1
    assert len(prms["2"]) == 2
    assert len(prms["3"]) == 2
    prms = aipy.scripting.parse_prms("a=(b/c)")
    assert len(prms["a"]) == 2



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Aaron Parsons
# Licensed under the GPLv3

import pytest
import aipy.healpix as ah
import numpy as np


def test_init():
    """Test initializing a aipy._alm object"""
    alm = ah.Alm(10, 5)
    assert alm.lmax() == 10
    assert alm.mmax() == 5
    with pytest.raises(Exception):
        ah.Alm(5, 10)

    return


def test_get_data():
    alm = ah.Alm(1, 1)
    data = alm.get_data()
    assert data.size == 3

    return


def test_set_data():
    alm = ah.Alm(1, 1)
    data = np.array([1 + 1j, 2 + 2j, 3 + 3j], dtype=np.complex128)
    alm.set_data(data)
    assert np.allclose(alm.get_data(), data)

    return


def test_set_to_zero():
    alm = ah.Alm(3, 3)
    assert np.allclose(alm.get_data(), 0.0)
    return


def test_get_set():
    alm = ah.Alm(1, 1)
    data = np.array([1 + 1j, 2 + 2j, 3 + 3j], dtype=np.complex128)
    alm.set_data(data)
    assert alm[0, 0] == data[0]
    assert alm[1, 0] == data[1]
    assert alm[1, 1] == data[2]

    alm[1, 1] = 4 + 4j
    assert alm[1, 1] == 4 + 4j

    return


def test_to_map():
    alm = ah.Alm(1, 1)
    alm[0, 0] = 1
    d = alm.to_map(32, "RING")
    assert np.allclose(d, np.ones(12 * 32 ** 2, dtype=np.float32) * 0.2820948)

    return


def test_from_map():
    alm = ah.Alm(1, 1)
    d = np.ones(12 * 32 ** 2, dtype=np.float32)
    alm.from_map(d, 3)
    assert np.allclose(alm[0, 0], 3.5449077018110313)
    assert np.allclose(alm[1, 0], 0.0)
    assert np.allclose(alm[1, 1], 0.0)

    return



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

import unittest, aipy.src as src

class TestSrc(unittest.TestCase):
    def test_get_catalog(self):
        """Test setting up a catalog"""
        cat = src.get_catalog(srcs=['J0535+220'])
        self.assertTrue(cat.has_key('J0535+220'))
        s = cat['J0535+220']
        self.assertEqual(s._jys, 1883.)
        self.assertEqual(s.index, -99.)
        cat = src.get_catalog(srcs=['J0535+220'], catalogs=[])
        self.assertEqual(len(cat.keys()), 0)

class TestSuite(unittest.TestSuite):
    """A unittest.TestSuite class which contains all of the aipy.src unit tests."""

    def __init__(self):
        unittest.TestSuite.__init__(self)

        loader = unittest.TestLoader()
        self.addTest(loader.loadTestsFromTestCase(TestSrc))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

"""
A script for dividing out the passband, primary beam, and/or source spectrum
scaling.  When dividing by a primary beam or source spectrum, it is recommended
a single source have been isolated in the data set.

Author: Aaron Parsons
"""

from __future__ import print_function, division, absolute_import

import aipy as a, numpy as np, os, sys, optparse, pickle
from matplotlib import pylab as p

o = optparse.OptionParser()
o.set_usage('flux_cal.py [options] *.uv')
o.set_description(__doc__)
a.scripting.add_standard_options(o, src=True, cal=True)
o.add_option('-b', '--beam', dest='beam', action='store_true',
    help='Normalize by the primary beam response in the direction of the specified source.')
o.add_option('-p', '--passband', dest='passband', action='store_true',
    help='Normalize by the passband response.')
o.add_option('-f', '--srcflux', dest='srcflux', action='store_true',
    help='Normalize by the spectrum of the specified source.')
opts, args = o.parse_args(sys.argv[1:])

assert(not (opts.src is None and (opts.beam or opts.srcflux)))
uv = a.miriad.UV(args[0])
freqs = a.cal.get_freqs(uv['sdf'], uv['sfreq'], uv['nchan'])
aa = a.cal.get_aa(opts.cal, uv['sdf'], uv['sfreq'], uv['nchan'])
del(uv)

if opts.srcflux:
    srclist,cutoff,catalogs = a.scripting.parse_srcs(opts.src, opts.cat)
    cat = a.cal.get_catalog(opts.cal, srclist, cutoff, catalogs)
    s = cat.values()[0]
    print('Calibrating for source with', end='')
    print('strength', s._jys, end='')
    print('measured at', s.mfreq, 'GHz', end='')
    print('with index', s.index)
    src_spec = None
else: src_spec = 1

curtime = None
def mfunc(uv, p, d, f):
    global curtime, src_spec
    uvw,t,(i,j) = p
    pol = a.miriad.pol2str[uv['pol']]
    aa.set_active_pol(pol)
    if t != curtime:
        curtime = t
        aa.set_jultime(t)
        if opts.srcflux:
            s.compute(aa)
            src_spec = s.get_jys()
            s_eq = cat.get_crds('eq', ncrd=3)
            aa.sim_cache(s_eq)
    if opts.passband:
        passband = aa.passband(i,j)
    else: passband = 1
    if opts.beam: bm_resp = aa.bm_response(i,j).squeeze()
    else: bm_resp = 1
    gain = passband * bm_resp * src_spec
    d /= np.where(gain == 0, 1, gain)
    return p, d, f

ext = '.'
if opts.passband: ext += 'p'
if opts.beam: ext += 'b'
if opts.srcflux: ext += 'f'
for filename in args:
    uvofile = filename + ext
    print(filename,'->',uvofile)
    if os.path.exists(uvofile):
        print('File exists: skipping')
        continue
    uvi = a.miriad.UV(filename)
    uvo = a.miriad.UV(uvofile, status='new')
    uvo.init_from_uv(uvi)
    uvo.pipe(uvi, mfunc=mfunc, raw=True,
        append2hist='FLUXCAL: srcs=%s passband=%s beam=%s srcflux=%s\n' % \
        (opts.src, opts.passband, opts.beam, opts.srcflux))



# -*- coding: utf-8 -*-
# Copyright (c) 2009 Aaron Parsons
# Licensed under the GPLv3

import pytest
import ephem
import random
import aipy
import numpy as np


def test_convert_typecheck():
    """Test coordinate conversion types"""
    crd2, crd3 = (0, 0), (1, 0, 0)
    for sys in aipy.coord.sys_dict:
        ans2 = aipy.coord.convert(crd2, sys, sys)
        assert crd2[0] == ans2[0]
        assert crd2[1] == ans2[1]

        ans3 = aipy.coord.convert(crd3, sys, sys)
        assert crd2[0] == ans3[0]
        assert crd2[1] == ans3[1]

        with pytest.raises(KeyError):
            aipy.coord.convert(crd2, "bad", "bad")

    return


def test_convert_precession():
    """Test coordinate precessions for accuracy"""
    crdpairs = [
        [("0:00", "0:00"), ("00:02:33.77", "00:16:42.1")],
        [("6:00", "0:00"), ("06:02:33.75", "-00:00:05.6")],
        [("0:00", "89:00"), ("00:03:03.75", "+89:16:41.7")],
    ]
    for b1950, j2000 in crdpairs:
        c1 = aipy.coord.convert(
            b1950, "eq", "eq", iepoch=ephem.B1950, oepoch=ephem.J2000
        )
        c2 = aipy.coord.convert(
            j2000, "eq", "eq", iepoch=ephem.J2000, oepoch=ephem.B1950
        )
        c1_ck = ephem.Equatorial(j2000[0], j2000[1], epoch=ephem.J2000).get()
        c2_ck = ephem.Equatorial(b1950[0], b1950[1], epoch=ephem.B1950).get()
        assert np.allclose(c1[0], c1_ck[0], atol=1e-4)
        assert np.allclose(c1[1], c1_ck[1], atol=1e-4)
        assert np.allclose(c2[0], c2_ck[0], atol=1e-4)
        assert np.allclose(c2[1], c2_ck[1], atol=1e-4)

    return


def test_convert_crdsys():
    """Test coordinate conversions for accuracy"""
    eq_ec_ga = [
        [
            ("19:59:28.3566", "40:44:02.096"),
            ("317.7054323", "59.3254895"),
            ("76.1898379", "5.7554756"),
        ],
        [
            ("12:30:49.4233", "12:23:28.043"),
            ("182.0592608", "14.4166861"),
            ("283.7777978", "74.4911308"),
        ],
        [
            ("13:25:27.6152", "-43:01:08.805"),
            ("217.1433477", "-31.3319020"),
            ("309.5158743", "19.4173247"),
        ],
    ]
    for eq, ec, ga in eq_ec_ga:
        eq_ec = aipy.coord.convert(eq, "eq", "ec")
        eq_ga = aipy.coord.convert(eq, "eq", "ga")
        ec_eq = aipy.coord.convert(ec, "ec", "eq")
        ec_ga = aipy.coord.convert(ec, "ec", "ga")
        ga_eq = aipy.coord.convert(ga, "ga", "eq")
        ga_ec = aipy.coord.convert(ga, "ga", "ec")
        eq_ck = ephem.Equatorial(eq[0], eq[1], epoch=ephem.J2000).get()
        ec_ck = ephem.Ecliptic(ec[0], ec[1], epoch=ephem.J2000).get()
        ga_ck = ephem.Galactic(ga[0], ga[1], epoch=ephem.J2000).get()

        assert np.allclose(eq_ec[0], ec_ck[0], atol=1e-4)
        assert np.allclose(eq_ec[1], ec_ck[1], atol=1e-4)
        assert np.allclose(eq_ga[0], ga_ck[0], atol=1e-4)
        assert np.allclose(eq_ga[1], ga_ck[1], atol=1e-4)
        assert np.allclose(ec_eq[0], eq_ck[0], atol=1e-4)
        assert np.allclose(ec_eq[1], eq_ck[1], atol=1e-4)
        assert np.allclose(ec_ga[0], ga_ck[0], atol=1e-4)
        assert np.allclose(ec_ga[1], ga_ck[1], atol=1e-4)
        assert np.allclose(ga_eq[0], eq_ck[0], atol=1e-4)
        assert np.allclose(ga_eq[1], eq_ck[1], atol=1e-4)
        assert np.allclose(ga_ec[0], ec_ck[0], atol=1e-4)
        assert np.allclose(ga_ec[1], ec_ck[1], atol=1e-4)

    return


def test_convert_m_shape():
    """Test conversion matrices for shape preservation"""
    for c1 in aipy.coord.sys_dict:
        for c2 in aipy.coord.sys_dict:
            mat = aipy.coord.convert_m(c1, c2)
            assert len(mat.shape) == 2
            assert mat.shape[0] == 3
            assert mat.shape[1] == 3

    return


def test_convert_m_diagonal():
    """Test conversion matrices for normalcy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    for c1 in aipy.coord.sys_dict:
        for c2 in aipy.coord.sys_dict:
            m1 = aipy.coord.convert_m(c1, c2)
            m2 = aipy.coord.convert_m(c2, c1)
            mat = np.round(np.dot(m1, m2), 10)
            assert np.allclose(mat, diag)

    return


def test_rot_m_shape():
    """Test rotation matrices for shape preservation"""
    mat = aipy.coord.rot_m(0, np.array([1.0, 0, 0]))
    assert len(mat.shape) == 2
    assert mat.shape[0] == 3
    assert mat.shape[1] == 3

    mat = aipy.coord.rot_m(np.array([0.0, 0.0]), np.array([[1.0, 0, 0], [1.0, 0, 0]]))
    assert len(mat.shape) == 3
    assert mat.shape[0] == 2
    assert mat.shape[1] == 3
    assert mat.shape[2] == 3

    return


def test_rot_m_accuracy():
    """Test rotation matricies for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = np.round(aipy.coord.rot_m(2 * np.pi, np.array([1.0, 0, 0])), 10)
    assert np.allclose(mat, diag)

    mat = np.round(aipy.coord.rot_m(np.pi / 2, np.array([0.0, 0, 1])), 10)
    assert np.allclose(mat, np.array([-e2, e1, e3]))

    mat = np.round(aipy.coord.rot_m(np.pi / 2, np.array([0, 1, 0])), 10)
    assert np.allclose(mat, np.array([e3, e2, -e1]))

    mat = np.round(aipy.coord.rot_m(np.pi / 2, np.array([1, 0, 0])), 10)
    assert np.allclose(mat, np.array([e1, -e3, e2]))

    return


def test_xyz2thphi_shape():
    """Test the x,y,z to theta,phi conversion for shape preservation"""
    mat = aipy.coord.xyz2thphi((0, 0, 1))
    assert len(mat.shape) == 1
    assert mat.shape[0] == 2

    mat = aipy.coord.xyz2thphi(
        (np.array([0.0, 0, 0]), np.array([0.0, 0, 0]), np.array([1.0, 1, 1]))
    )

    assert len(mat.shape) == 2
    assert mat.shape[0] == 2
    assert mat.shape[1] == 3

    return


def test_xyz2thphi_accuracy():
    """Test the x,y,z to theta,phi conversion for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = np.round(aipy.coord.xyz2thphi(e3), 10)
    assert np.allclose(mat, np.array([0.0, 0]))

    mat = np.round(aipy.coord.xyz2thphi(e1), 10)
    assert np.allclose(mat, np.round(np.array([np.pi / 2, 0]), 10))

    mat = np.round(aipy.coord.xyz2thphi(e2), 10)
    assert np.allclose(mat, np.round(np.array([np.pi / 2, np.pi / 2]), 10))

    return


def test_thphi2xyz_shape():
    """Test the theta,phi to x,y,z conversion for shape preservation"""
    mat = aipy.coord.thphi2xyz((0, 0))
    assert len(mat.shape) == 1
    assert mat.shape[0] == 3

    mat = aipy.coord.thphi2xyz((np.array([0.0, 0]), np.array([0.0, 0])))
    assert len(mat.shape) == 2
    assert mat.shape[0] == 3
    assert mat.shape[1] == 2

    return


def test_thphi2xyz_accuracy():
    """Test the theta,phi to x,y,z conversion for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = np.round(aipy.coord.thphi2xyz((0, 0)), 10)
    assert np.allclose(mat, e3)

    mat = np.round(aipy.coord.thphi2xyz((np.pi / 2, 0)), 10)
    assert np.allclose(mat, e1)

    mat = np.round(aipy.coord.thphi2xyz((np.pi / 2, np.pi / 2)), 10)
    assert np.allclose(mat, e2)

    return


def test_eq2radec_shape():
    """Test the equatorial to ra,dec conversion for shape preservation"""
    mat = aipy.coord.eq2radec((0, 0, 1))
    assert len(mat.shape) == 1
    assert mat.shape[0] == 2

    mat = aipy.coord.eq2radec(
        (np.array([0.0, 0, 0]), np.array([0.0, 0, 0]), np.array([1.0, 1, 1]))
    )
    assert len(mat.shape) == 2
    assert mat.shape[0] == 2
    assert mat.shape[1] == 3

    return


def test_eq2radec_accuracy():
    """Test the equatorial to ra,dec conversion for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = np.round(aipy.coord.eq2radec(e3), 10)
    assert np.allclose(mat, np.round(np.array([0.0, np.pi / 2]), 10))

    mat = np.round(aipy.coord.eq2radec(e1), 10)
    assert np.allclose(mat, np.round(np.array([0, 0]), 10))

    mat = np.round(aipy.coord.eq2radec(e2), 10)
    assert np.allclose(mat, np.round(np.array([np.pi / 2, 0]), 10))

    return


def test_radec2eq_shape():
    """Test the ra,dec to equatorial conversion for shape preservation"""
    mat = aipy.coord.radec2eq((0, 0))
    assert len(mat.shape) == 1
    assert mat.shape[0] == 3

    mat = aipy.coord.radec2eq((np.array([0.0, 0]), np.array([0.0, 0])))
    assert len(mat.shape) == 2
    assert mat.shape[0] == 3
    assert mat.shape[1] == 2

    return


def test_radec2eq_accuracy():
    """Test the ra,dec to equatorial conversion for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = np.round(aipy.coord.radec2eq((0, 0)), 10)
    assert np.allclose(mat, e1)

    mat = np.round(aipy.coord.radec2eq((np.pi / 2, 0)), 10)
    assert np.allclose(mat, e2)

    mat = np.round(aipy.coord.radec2eq((np.pi / 2, np.pi / 2)), 10)
    assert np.allclose(mat, e3)

    return


def test_latlong2xyz_shape():
    """Test the lat,long to x,y,z conversion for shape preservation"""
    mat = aipy.coord.latlong2xyz((0, 0))
    assert len(mat.shape) == 1
    assert mat.shape[0] == 3

    mat = aipy.coord.latlong2xyz((np.array([0.0, 0]), np.array([0.0, 0])))
    assert len(mat.shape) == 2
    assert mat.shape[0] == 3
    assert mat.shape[1] == 2

    return


def test_latlong2xyz_accuracy():
    """Test the lat,long to x,y,z conversion for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = np.round(aipy.coord.latlong2xyz((0, 0)), 10)
    assert np.allclose(mat, e1)

    mat = np.round(aipy.coord.latlong2xyz((np.pi / 2, 0)), 10)
    assert np.allclose(mat, e3)

    mat = np.round(aipy.coord.latlong2xyz((0, np.pi / 2)), 10)
    assert np.allclose(mat, e2)

    return


def test_top2azalt_shape():
    """Test the x,y,z to az,alt conversion for shape preservation"""
    mat = aipy.coord.top2azalt((0, 0, 1))
    assert len(mat.shape) == 1
    assert mat.shape[0] == 2

    mat = aipy.coord.top2azalt(
        (np.array([0.0, 0, 0]), np.array([0.0, 0, 0]), np.array([1.0, 1, 1]))
    )
    assert len(mat.shape) == 2
    assert mat.shape[0] == 2
    assert mat.shape[1] == 3

    return


def test_top2azalt_accuracy():
    """Test the x,y,z to az,alt conversion for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = aipy.coord.top2azalt(e3)
    assert np.allclose(mat[1], np.pi / 2, atol=1e-10)

    mat = np.round(aipy.coord.top2azalt(e1), 10)
    assert np.allclose(mat, np.round(np.array([np.pi / 2, 0]), 10))

    mat = np.round(aipy.coord.top2azalt(e2), 10)
    assert np.allclose(mat, np.round(np.array([0, 0]), 10))

    return


def test_azalt2top_shape():
    """Test the az,alt to x,y,z conversion for shape preservation"""
    mat = aipy.coord.azalt2top((0, 0))
    assert len(mat.shape) == 1
    assert mat.shape[0] == 3

    mat = aipy.coord.azalt2top((np.array([0.0, 0]), np.array([0.0, 0])))
    assert len(mat.shape) == 2
    assert mat.shape[0] == 3
    assert mat.shape[1] == 2

    return


def test_azalt2top_accuracy():
    """Test the az,alt to x,y,z conversion for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = np.round(aipy.coord.azalt2top((0, 0)), 10)
    assert np.allclose(mat, e2)

    mat = np.round(aipy.coord.azalt2top((np.pi / 2, 0)), 10)
    assert np.allclose(mat, e1)

    mat = np.round(aipy.coord.azalt2top((0, np.pi / 2)), 10)
    assert np.allclose(mat, e3)

    return


def test_eq2top_m_shape():
    """Test the equatorial/x,y,z rotation matrix for shape preservation"""
    mat = aipy.coord.eq2top_m(0, 0)
    assert len(mat.shape) == 2
    assert mat.shape[0] == 3
    assert mat.shape[1] == 3

    mat = aipy.coord.eq2top_m(np.array([0.0, 0]), np.array([0.0, 0]))
    assert len(mat.shape) == 3
    assert mat.shape[0] == 2
    assert mat.shape[1] == 3
    assert mat.shape[2] == 3

    return


def test_eq2top_m_accuracy():
    """Test the equatorial/x,y,z rotation matrix for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = np.round(aipy.coord.eq2top_m(0, 0), 10)
    assert np.allclose(mat, np.array([e2, e3, e1]))

    mat = np.round(aipy.coord.eq2top_m(-np.pi / 2, 0.0), 10)
    assert np.allclose(mat, np.array([-e1, e3, e2]))

    mat = np.round(aipy.coord.eq2top_m(0, np.pi / 2), 10)
    assert np.allclose(mat, np.array([e2, -e1, e3]))

    return


def test_top2eq_m_shape():
    """Test the x,y,z/equatorial rotation matrix for shape preservation"""
    mat = aipy.coord.top2eq_m(0, 0)
    assert len(mat.shape) == 2
    assert mat.shape[0] == 3
    assert mat.shape[1] == 3

    mat = aipy.coord.top2eq_m(np.array([0.0, 0]), np.array([0.0, 0]))
    assert len(mat.shape) == 3
    assert mat.shape[0] == 2
    assert mat.shape[1] == 3
    assert mat.shape[2] == 3

    return


def test_top2eq_m_accuracy():
    """Test the x,y,z/equatorial rotation matrix for accuracy"""
    diag = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float64)
    e1, e2, e3 = diag
    mat = np.round(aipy.coord.top2eq_m(0, 0), 10)
    assert np.allclose(mat, np.array([e3, e1, e2]))

    mat = np.round(aipy.coord.top2eq_m(-np.pi / 2, 0.0), 10)
    assert np.allclose(mat, np.array([-e1, e3, e2]))

    mat = np.round(aipy.coord.top2eq_m(0, np.pi / 2), 10)
    assert np.allclose(mat, np.array([-e2, e1, e3]))

    return



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Aaron Parsons
# Licensed under the GLPv3

import pytest
import random
import ephem
import aipy
import numpy as np

DIM = 128


def test_clean2dc():
    res = np.zeros((DIM, DIM), dtype=np.complex64)
    ker = np.zeros((DIM, DIM), dtype=np.complex64)
    mdl = np.zeros((DIM, DIM), dtype=np.complex64)
    area = np.zeros((DIM, DIM), dtype=np.int64)
    with pytest.raises(ValueError):
        aipy._deconv.clean(res, ker, mdl, area.astype(np.float64))
    ker[0, 0] = 1.0
    res[0, 0] = 1.0
    res[5, 5] = 1.0
    area[:4, :4] = 1
    rv = aipy._deconv.clean(res, ker, mdl, area, tol=1e-8)
    assert np.allclose(res[0, 0], 0, atol=1e-3)
    assert res[5, 5] == 1

    return


def test_clean2dr():
    res = np.zeros((DIM, DIM), dtype=np.float32)
    ker = np.zeros((DIM, DIM), dtype=np.float32)
    mdl = np.zeros((DIM, DIM), dtype=np.float32)
    area = np.zeros((DIM, DIM), dtype=np.int64)
    with pytest.raises(ValueError):
        aipy._deconv.clean(res, ker, mdl, area.astype(np.float32))
    ker[0, 0] = 1.0
    res[0, 0] = 1.0
    res[5, 5] = 1.0
    area[:4, :4] = 1
    rv = aipy._deconv.clean(res, ker, mdl, area, tol=1e-8)
    assert np.allclose(res[0, 0], 0, atol=1e-3)

    return


def test_clean1dc():
    res = np.zeros((DIM,), dtype=np.complex64)
    ker = np.zeros((DIM,), dtype=np.complex64)
    mdl = np.zeros((DIM,), dtype=np.complex64)
    area = np.zeros((DIM,), dtype=np.int64)
    with pytest.raises(ValueError):
        aipy._deconv.clean(res, ker, mdl, area.astype(np.float32))
    ker[0] = 1.0
    res[0] = 1.0
    res[5] = 1.0
    area[:4] = 1
    rv = aipy._deconv.clean(res, ker, mdl, area, tol=1e-8)
    assert np.allclose(res[0], 0, atol=1e-3)
    assert res[5] == 1

    return


def test_clean1dr():
    res = np.zeros((DIM,), dtype=np.float32)
    ker = np.zeros((DIM,), dtype=np.float32)
    mdl = np.zeros((DIM,), dtype=np.float32)
    area = np.zeros((DIM,), dtype=np.int64)
    with pytest.raises(ValueError):
        aipy._deconv.clean(res, ker, mdl, area.astype(np.float32))
    ker[0] = 1.0
    res[0] = 1.0
    res[5] = 1.0
    area[:4] = 1
    rv = aipy._deconv.clean(res, ker, mdl, area, tol=1e-8)
    assert np.allclose(res[0], 0, atol=1e-3)
    assert res[5] == 1

    return


def test_clean2d_stop_if_div():
    DIM1, DIM2 = 1000, 250
    dim = np.random.normal(size=(DIM1, DIM2))
    dbm = np.random.normal(size=(DIM1, DIM2))
    mdl = np.zeros(dim.shape, dtype=dim.dtype)
    area = np.ones(dim.shape, dtype=np.int64)
    init_val = dim[0, 0]
    rv = aipy._deconv.clean(
        dim, dbm, mdl, area, gain=0.1, tol=1e-2, stop_if_div=0, maxiter=100
    )
    assert np.allclose(dim[0, 0], init_val, atol=1e-3)

    return



#! /usr/bin/env python

from __future__ import absolute_import, division, print_function

from setuptools import setup, Extension

import os, glob, numpy, subprocess, sys

PY2 = sys.version_info.major < 3
if PY2:
    MATPLOTLIB_DEP = 'matplotlib<3'
    ASTROPY_DEP = 'astropy>=1.0'
else:
    MATPLOTLIB_DEP = 'matplotlib'
    ASTROPY_DEP = 'astropy>=3.0'

def get_description():
    def get_description_lines():
        seen_desc = False

        with open('README.md') as f:
            for line in f:
                if seen_desc:
                    if line.startswith('##'):
                        break
                    line = line.strip()
                    if len(line):
                        yield line
                elif line.startswith('## Description'):
                    seen_desc = True

    return ' '.join(get_description_lines())

def indir(path, files):
    return [os.path.join(path, f) for f in files]

global_macros = [('NPY_NO_DEPRECATED_API', 'NPY_1_7_API_VERSION')]

setup(
    name = 'aipy',
    use_setuptools_scm={
        "write_to": "aipy/_version.py",
        "parentdir_prefix_version": "aipy-",
        "fallback_version": "0.0.0",
    },
    description = 'Astronomical Interferometry in PYthon',
    long_description = get_description(),
    license = 'GPL',
    author = 'Aaron Parsons',
    author_email = 'aparsons@berkeley.edu',
    url = 'http://github.com/HERA-Team/aipy',
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Topic :: Scientific/Engineering :: Astronomy',
    ],
    setup_requires = [
        'numpy>=1.2',
        'setuptools_scm',
    ],
    install_requires = [
        ASTROPY_DEP,
        'healpy>=1.11',
        MATPLOTLIB_DEP,
        'numpy>=1.2',
        'ephem>=3.7.3.2',
        'scipy>=0.19',
    ],
    extras_require = {
        'dev': [
            'pytest',
            'pytest-cov'
        ]
    },
    package_dir = {'aipy':'aipy', 'aipy._src':'aipy/_src'},
    packages = ['aipy', 'aipy._src'],
    ext_modules = [
        Extension('aipy._miriad', ['aipy/_miriad/miriad_wrap.cpp'] + \
            indir('aipy/_miriad/mir', ['uvio.c','hio.c','pack.c','bug.c',
                'dio.c','headio.c','maskio.c']),
            define_macros = global_macros,
            include_dirs = [numpy.get_include(), 'aipy/_miriad',
                'aipy/_miriad/mir', 'aipy/_common']),
        Extension('aipy._deconv', ['aipy/_deconv/deconv.cpp'],
            define_macros = global_macros,
            include_dirs = [numpy.get_include(), 'aipy/_common']),
        #Extension('aipy._img', ['aipy/_img/img.cpp'],
        #    include_dirs = [numpy.get_include()]),
        Extension('aipy._dsp', ['aipy/_dsp/dsp.c', 'aipy/_dsp/grid/grid.c'],
            define_macros = global_macros,
            include_dirs = [numpy.get_include(), 'aipy/_dsp', 'aipy/_dsp/grid', 'aipy/_common']),
        Extension('aipy.utils', ['aipy/utils/utils.cpp'],
            define_macros = global_macros,
            include_dirs = [numpy.get_include(), 'aipy/_common']),
    ],
    scripts = glob.glob('scripts/*'),

    include_package_data = True,
    zip_safe = False,
    test_suite = "tests.aipy_test.TestSuite",
)



#! /usr/bin/env python

from __future__ import print_function, division, absolute_import

import aipy as a, numpy as n
from matplotlib import pylab as p

img = n.array([0,0,0,4,6,4,0,0,-2,-3,-2,0], dtype=n.float)
ker = n.array([3,2,0,0,0,0,0,0,0,0,0,2], dtype=n.float)

print('REAL TEST:')
print('img', img)
print('ker', ker)
cln, info = a.deconv.clean(img, ker)
print('cln', cln)
print(info)
print('-----------------------------------------------------------------')

img = n.array([0,0,0,4j,6j,4j,0,0,2+2j,3+3j,2+2j,0], dtype=n.complex)
ker = n.array([3,2,0,0,0,0,0,0,0,0,0,2], dtype=n.complex)

print('CMPLX TEST:')
print('img', img)
print('ker', ker)
mdl, info = a.deconv.clean(img, ker)
print('cln', cln)
print(info)
print('-----------------------------------------------------------------')

SIZE = 16
img = n.zeros((SIZE,), n.complex)
img[5] = 1+2j
img[10] = 3+4j
ker = n.zeros((SIZE,), n.complex)
ker[0] = 2j
ker[1] = 1+1j; ker[-1] = -1-1j
ker[2] = .5+.5j; ker[-2] = -.5-.5j
d1d = n.fft.ifft(n.fft.fft(img) * n.fft.fft(ker))
print('TEST:')
print('img', d1d)
print('ker', ker)
cln, info = a.deconv.clean(d1d, ker, maxiter=200)
print('cln', cln)
print(info)

p.plot(n.abs(cln), 'b')
p.plot(n.abs(img), 'k.')
p.plot(n.abs(img - cln), 'r')
p.show()
